//+------------------------------------------------------------------+
//|                                        Custom Moving Average.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
// Translated into decent english by Sanford Staab
//
#property copyright "Copyright  2004, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

// This indicator will show up in the same window as the currency
// is plotted in.
// If you want this in a separate window (ie appearing just below
// the currency graph) use indicator_separate_window
#property indicator_chart_window

// This tells the compiler that this indicator has only one buffer
// (ie one line) drawn for it.
// Note that the system does all the work of iterating through these
// buffers and drawing them for you.  More on that later...
#property indicator_buffers 1

// Tell the system that the default color for the first (and only) 
// indicator is red.
#property indicator_color1 Red

//---- indicator parameters
// By convention an extern declaration for indicators means these are
// parameters that will appear in the indicator properties window.
// (Its too bad that we can't define some help text to let user's better
// understand what these variable mean to the indicator.
// In this case, the user needs to know what each method code means. 
// There is no way for him to know unless he wrote this indicator.)
extern int MA_NumberOfSamples=13;
extern int MA_ShiftRightBarCount=0;
extern int MA_MethodCode=0;

//---- indicator buffers
// This specifies the global variable used to hold/access the first (only)
// indicator.  
double ExtMapBuffer[];
//----
// All bars need to be calculated when we start the indicator so set 
// this to 0 initially. We are defining this globaly so that it 
// can be used by each of the calculate functions.
int UnchangedBarCount=0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
// This gets called whenever the indicator is loaded or reloaded
// or reinitialized.  Here you need to clear out anything that might
// effect the proper operation of the start() function.  
int init()
{
   int    iBarDrawBegin; // bar index of where we start drawing the indicator.
   string short_name; // a short name for the Moving Average type
                      // EMA = Exponential Moving Average
                      // SMMA = Smoothed M? Moving Average
                      // LWMA = Long Winded Moving Avarage? 
//---- drawing settings
// Here is where we tell the system about how to draw our first (and only)
// indicator.  Index 0 referrs to the first one.  The system supports up
// to 8 simultaneous indicator arrays per indicator script. (last index=7)
//
// Draw this indicator as a line for me
   SetIndexStyle(0, DRAW_LINE);
// Shift this indicator to the right MA_ShiftRightBarCount bars when rendering.
// Note that a negative MA_ShiftRightBarCount would shift it to the left.
   SetIndexShift(0, MA_ShiftRightBarCount);
// Render the value of the indicator to the proper number of digits of precision
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS));
   
// ensure the user didn't set the number of samples incorrectly
   if (MA_NumberOfSamples<2) 
   {
      MA_NumberOfSamples=13;
   }
// we only need to draw the last number of sample bars so calc that here.   
   iBarDrawBegin=MA_NumberOfSamples - 1;
   
//---- set the indicator MA method short name basd on the method code
   switch(MA_MethodCode)
   {
      case 1 : 
         short_name="EMA(";  
         // this ma always requires a full calculation
         iBarDrawBegin=0; 
         break;
         
      case 2 : 
         short_name="SMMA("; 
         break;
         
      case 3 : 
         short_name="LWMA("; 
         break;
         
      default :
         MA_MethodCode=0;     // fix any user input bobos
         short_name="SMA(";
   }
   // now tell the system the name of our indicator.
   // (This doesn't seem to work as what I see when mousing over
   // the MA line is "MyMoving Averages Time... Value1...")
   IndicatorShortName(short_name + MA_NumberOfSamples + ")");
   // tell the system to have us calculate from the iBarDrawBegin point.
   SetIndexDrawBegin(0, iBarDrawBegin);
   // tell the system to map our internal array variable to the first
   // indicator
   SetIndexBuffer(0, ExtMapBuffer);

   return(0); // I have no idea why this function returns 0. Docs don't say.
}
//+------------------------------------------------------------------+
//| This is called each time a new bar period starts                 |
//+------------------------------------------------------------------+
int start()
{
   // if we don't have enough bars to calc this average, skip it.
   if (Bars <= MA_NumberOfSamples) 
   {
      return(0);
   }
   
   // set this for use by the calc functions (why not pass this in?)
   UnchangedBarCount = IndicatorCounted();  // Should be named CountOfUnchangedBarsLast()
   
   //---- check for possible errors (docs say nothing about what -1 would mean here.
   if (UnchangedBarCount < 0) 
   {
      // lets put out what the last error was for posterity.
      Print("Error=" + GetLastError());
      // docs say nothing about what this return value means.
      return(-1);
   }
//---- last counted bar will be recounted
   if (UnchangedBarCount > 0) 
   {
      //
      // Note that here is how the bar indexes are over time:
      // [10][9][8][7][6][5][4][3][2][1][0]
      //   ^                             ^
      // Oldest bar                 Most recent bar
      //
      // by decrementing the count we reduce the number of bars we will recalc
      //
      UnchangedBarCount--;
   }
//---- call the appropriate claclulating function based on our Method Code.
   switch(MA_MethodCode)
   {
      case 0 : 
         sma();  
         break;
         
      case 1 : 
         ema();  
         break;
         
      case 2 : 
         smma(); 
         break;
         
      case 3 : 
         lwma(); 
         break;
   }

   return(0); // docs say nothing about the return code for start()
}


//+------------------------------------------------------------------+
//| Simple Moving Average calculator                                 |
//+------------------------------------------------------------------+
void sma()
  {
   double sum;
   int   pos;
   //---- initial accumulation
   pos = Bars - UnchangedBarCount - 1;
   if (pos < MA_NumberOfSamples) 
   {
      pos = MA_NumberOfSamples;
   }
   for(sum = 0, int i = 1; i < MA_NumberOfSamples; i++, pos--)
   {
      // Here's what's happening:
      // say: Bars = 15, UnchangedBarCount = 2      
      // [14][13][12][11][10][9][8][7][6][5][4][3][2][1][0]
      //           ^---MA_NumberOfSamples - 1---^
      //           ^ pos starts here
      //                             pos ends here ^  
      //
      // The idea is, we are going to add up the most recent
      // 10 bar close values and use that for our most recent
      // MA calc.
      sum += Close[pos];
   }
   
   // now loop to calc each unchanged bar
   while(pos >= 0)
   {
      // add up the close value to the sum
      sum += Close[pos];
      // store the averate in our indicator buffer
      ExtMapBuffer[pos] = sum / MA_NumberOfSamples;
      // remove the oldest close value from the sum
      // this effectively moves our sum range to the next
      // more-recent bar
	   sum -= Close[pos + MA_NumberOfSamples - 1];
	   // update the position to the next more recent spot
 	   pos--;
   }

   // If there were no unchanged bars, the oldest MA_NumberOfSamples
   // bars are bogus and should be zeroed out.
   if (UnchangedBarCount < 1)
   {
      for(i=1; i < MA_NumberOfSamples; i++) 
      {
         // though I can't find it in the documentation, a value of
         // 0 in the indicator buffer seems to mean NODRAW - it would
         // be nice to make this a constant.
         ExtMapBuffer[Bars - i] = 0;
      }
   }
}
//+------------------------------------------------------------------+
//| Exponential Moving Average                                       |
//+------------------------------------------------------------------+
void ema()
{
   double pr = 2.0 / (MA_NumberOfSamples + 1);
   int    pos = Bars - 2;
   
   if (UnchangedBarCount > 2) 
   {
      pos = Bars - UnchangedBarCount - 1;
   }
   while(pos >= 0)
   {
      if (pos == Bars - 2)
      { 
         ExtMapBuffer[pos+1] = Close[pos+1];
      }
      ExtMapBuffer[pos] = Close[pos] * pr + ExtMapBuffer[pos+1] * (1-pr);
 	   pos--;
   }
}
//+------------------------------------------------------------------+
//| Smoothed Moving Average                                          |
//+------------------------------------------------------------------+
void smma()
{
   double sum=0;
   int i;
   int k;
   int pos;

   pos = Bars - MA_NumberOfSamples;
   if (pos > Bars - UnchangedBarCount) 
   {
      pos = Bars - UnchangedBarCount;
   }
   while(pos>=0)
   {
      if (pos == Bars - MA_NumberOfSamples)
      {
         //---- initial accumulation
         for(i=0, k=pos; i < MA_NumberOfSamples; i++, k++)
         {
            sum += Close[k];
            //---- zero initial bars
            ExtMapBuffer[k] = 0;
         }
      }
      else 
      {
         sum = ExtMapBuffer[pos + 1] * (MA_NumberOfSamples - 1) + Close[pos];
      }
      ExtMapBuffer[pos] = sum / MA_NumberOfSamples;
 	   pos--;
   }
}
//+------------------------------------------------------------------+
//| Linear Weighted Moving Average                                   |
//+------------------------------------------------------------------+
void lwma()
{
   double sum=0.0,lsum=0.0;
   double price;
   int i;
   int weight = 0;
   int pos = Bars - UnchangedBarCount - 1;

   if (pos < MA_NumberOfSamples) 
   {
      pos = MA_NumberOfSamples;
   }
   for(i=1; i <= MA_NumberOfSamples; i++, pos--)
   {
      price = Close[pos];
      sum += price*i;
      lsum += price;
      weight += i;
   }

   pos++;
   i = pos + MA_NumberOfSamples;
   while(pos >= 0)
   {
      ExtMapBuffer[pos] = sum / weight;
      if (pos==0) 
         break;
      pos--;
      i--;
      price = Close[pos];
      sum = sum - lsum + price * MA_NumberOfSamples;
      lsum -= Close[i];
      lsum += price;
   }
   //---- zero initial bars
   if(UnchangedBarCount<1)
   {
      for(i=1; i < MA_NumberOfSamples; i++) 
      {
         ExtMapBuffer[Bars-i] = 0;
      }
   }
}


